% DP832 calibration script using a 34461A or DM3068
% v1.2 - 25-Feb-2016
%
%  LaurentR (EEVblog) 2015
% Portions  Agilent Technologies 2013 (See Matlab examples)
% http://www.keysight.com/main/techSupport.jspx?cc=US&lc=eng&nid=-536902435.1058513&pid=2270273&pageMode=OV
%
% Free for any use with no warranty, obligation or liability implied by 
% LaurentR or Agilent Technologies.
%
% All credits go to the work of TooOldForThis and ted572 (EEVblog)
% 
% Their work, the latest version of this script and comments can be found
% on the EEVblog DP832 thread:
% http://www.eevblog.com/forum/testgear/rigol-dp832-firmware-updates-and-bug-list/
%
% Release notes
% v1.0 (14-Jan-2015): - Initial release
% v1.1 (15-Feb-2015): - Various cleanup
%                     - Changed the date string function for older Matlab
%                       compatibility (thanks to EEVblog's Sparky).
%                     - Tentative Rigol DM3068 support using the suggestion
%                       by EEVblog's smgvbest to set CMDSet to AGILENT (untested)
%v1.2  (26-Feb-2016): - Added support for Keithley DMM7510
%                     - Added line terminations \n for instrument fprintf:s
%                     - General cleanup
%                     - Added query('*OPC?') in strategic places to sync
%                       readouts
%                     - Added averaging for DMM7510 directly on the
%                       instrument, set myAverage variable > 1 to enable
%                     - Added sample output to end of file
%                     - EEVblog member: Kazam
%
% Usage notes
% - Use at your own risk.
%   - Understand the script before using it.
%   - Do a dry run (default option) before actually storing calibration
% 
% - By default the script DOES NOT STORE CALIBRATION at the end.
%   - Once you are confident with the script and procedure, change the
%   simOnly variable to false.
%
% - A substantial warmup (>1h) of both devices is higly recommended.
%   - The DP832 temperature drift is noticeable, so calibrate at your usual
%   ambient temperature for best results.
%
% - This script requires MATLAB with the Instrument Control toolbox
% installed
%   - Tested with R2014b on Windows 7 64b
%
% - This script supports only the Rigol DP832.
%   - Tested with FW 00.01.13, NI VISA drivers and a USB connection
%   - This script can probably be used for other models, but changes are
%   likely required (e.g. to the calibration tables)
%
% - This script assumes an Agilent 34461A is used.
%   - Tested with FW 01.10, NI VISA drivers and a USB connection
%   - The 10A current range (and connector) are used for all current
%   measurements. There is a minor loss of accuracy, but it is simpler and
%   safer that way (and well within the DP832 accuracy specification).
%   Better current accuracy may be obtained by modifying the script to use
%   the 3A connector and Auto ranging to get the best accuracy as long
%   as the current is <3A.
%   - A 34401A should be able to use the script almost directly, but the
%   current calibration include a 3.2A step which is above the supported
%   range of the 34401A, so creativity is required.
%
% - Tentative support for the Rigol DM3068 is added in v1.1
%   - The support is entirely untested as I do not have a DM3068.
%   - Please report any success or failure to the EEVblog thread.
%   - Support is done using the CMDSet AGILENT function of the DM3068 which
%     supports all common 34401A voltage and current commands used in this
%     script.
%   - A basic check of the DM3068 Programming Guide has been done to ifdef
%     out any 34461A-only command (mostly temp and uptime commands).
%
% - The script assumes NI Visa drivers and 'DP832' and '34461A' Visa
% aliases for the 2 SCPI devices. Change as needed.
%
% - The script uses a pause of 2 seconds bewteen PSU commands and DMM
% readout. AFAICT, this is quite conservative. Current measurements may
% benefit from a higher pause time to allow more time for self-heating.
%
% - The script uses a sinhle 100PLC readout on the DMM. AFAICT, this is
% sufficient as the PSU is very stable. Multiple readouts with further
% averaging is possible.

% MAIN

function DP832Calib

    % Constants subject to change
    % Review/modify before using
    myPause = 1; % 2 second pause between sending value to PSU and reading DMM value
    myNplc = 12; % Max precision
    myChannels = 3; % Can put a smaller number for a dry run
    myAverage = 2; %Number of readings to average over for the DMM mesurement. Applies to the DMM7510.
    simOnly = false; % Set to false to actually store calibration at the end
    psuAddr = 'DP832'; % Set to the PSU Visa address or alias
    psuDrv = 'agilent'; % Set to 'agilent' if using the Agilent Visa drivers for the PSU
    dmmAddr = 'DMM7510'; % Set to the DMM Visa address or alias
    dmmDrv = 'agilent'; % Set to 'agilent' if using the Agilent Visa drivers for the DMM

    % Calibration tables
    DACV12 = [0.2 0.5 1.2 2 3.2 4.1 5.2 6.9 7.5 8.7 10.1 11.8 12.6 13.5 15 15.8 16.5 17.3 18.5 19.1 19.9 20.2 20.8 21.8 22.4 22.7 23.9 24.3 25.7 26.9 27.9 28.5 28.9 29.8 30.2 32];
    ADCV12 = [0 0.05 0.1 0.5 1 5 10 12.8 20 30 32];
    DACI12 = [0.1 0.25 0.5 0.8 1 1.25 1.5 1.75 1.9 2.15 2.35 2.5 2.75 3 3.2];
    ADCI12 = [0 0.01 0.1 1 2 3 3.2];
    DACV3 = [0.1 0.2 0.4 0.85 1.2 1.8 2.55 3.1 3.4 4.1 4.5 5 5.3];
    ADCV3 = [0 0.005 0.01 0.02 0.05 0.1 0.5 1 3 5 5.3];
    DACI3 = [0.1 0.5 1 1.25 1.5 1.75 2 2.25 2.5 2.75 3 3.2];
    ADCI3 = [0 0.1 1 2 3 3.2];

    newobjs = instrfind;
    if isempty(newobjs) == 0
        fclose(newobjs);
        delete(newobjs);
        clear newobjs
    end
    try
        if simOnly
            fprintf('NOTE: This script will NOT store the calibration into the PSU at the end.\n');
        else
            fprintf('NOTE: This script WILL STORE the calibration into the PSU at the end.\n');
        end
        fprintf('If this is not your intent, please abort and modify the value of the  "simOnly" variable.\n\n');
        myPsu = visa(psuDrv, psuAddr);
        fopen(myPsu);
        psuIdn = query(myPsu, '*IDN?');
        CheckVisaError(myPsu);
        fprintf('PSU: %s\n\n', strtrim(psuIdn));
        if isempty(strfind(psuIdn, 'DP832'))
            err = MException('Calib:BadInput', 'ERROR: Unsupported PSU');
            throw(err);
        end
        fprintf(myPsu, '*RST\n');   
        CheckVisaError(myPsu);
        
        myDmm = visa(dmmDrv, dmmAddr);
        fopen(myDmm);
        
        dmmIdn = query(myDmm, '*IDN?');
        fprintf(myDmm, '*RST\n');   
        CheckVisaError(myDmm);
        fprintf('DMM: %s\n\n', strtrim(dmmIdn));
        is34461A = ~isempty(strfind(dmmIdn, '34461A'));
        isDM3068 = ~isempty(strfind(dmmIdn, 'DM3068'));
        isDMM7510 = ~isempty(strfind(dmmIdn, 'DMM7510'));

        if ~is34461A && ~isDM3068 && ~isDMM7510
            err = MException('Calib:BadInput', 'ERROR: Unsupported DMM');
            throw(err);
        end
        fprintf(myDmm, '*RST\n');
        CheckVisaError(myDmm);
        if isDM3068
            fprintf(myDmm, 'CMDSet AGILENT');
            CheckVisaError(myDmm);
        end
        
        fprintf('Remove all leads from DMM and PSU to run self-tests\n');
        input('Press Enter\n');

        fprintf('Running PSU board self-test...\n');
        r = query(myPsu, 'SYSTem:SELF:TEST:BOARD?');
        CheckVisaError(myPsu);
        fprintf('PSU self-test result: %s\n', strtrim(r));
        r = str2double(query(myPsu, 'SYSTem:SELF:TEST:TEMP?'));
        CheckVisaError(myPsu);
        fprintf('PSU internal temp: %4.1fC\n\n', r);
                
        fprintf('Running DMM self-test...\n');
        r = query(myDmm, '*TST?');
        CheckVisaError(myDmm);
        fprintf('DMM self-test result: %s (0 = PASS, 1=FAIL)\n', strtrim(r));
        if is34461A
            r = str2double(query(myDmm, 'SYST:TEMP?'));
            CheckVisaError(myDmm);
            fprintf('DMM internal temp: %4.1fC\n', r);
            upt = query(myDmm, 'SYST:UPT?');
            CheckVisaError(myDmm);
            uptn = sscanf(upt, '%d,%d,%d,%d');
            fprintf('DMM uptime: %dd %dh %dm %ds\n', uptn(1), uptn(2), uptn(3), uptn(4));
        end
        
        if isDMM7510
            r = str2double(query(myDmm, 'SYST:TEMP:INT?'));
            CheckVisaError(myDmm);
            fprintf('DMM internal temp: %4.1fC\n', r);
        end

        for ch = myChannels
            if (ch==1) || (ch==2)
                DACV = DACV12;
                ADCV = ADCV12;
                DACI = DACI12;
                ADCI = ADCI12;
            else
                DACV = DACV3;
                ADCV = ADCV3;
                DACI = DACI3;
                ADCI = ADCI3;
            end

            fprintf('\nStarting channel %d\n\n', ch);
            fprintf(myPsu, 'CALibration:Start 11111,CH%d\n', ch);
            fprintf(myPsu, 'CALibration:Clear CH%d,ALL\n', ch);
            fprintf(myPsu, '*RST\n');
            CheckVisaError(myPsu);

            fprintf('Starting Voltage: connect the DMM VOLTAGE inputs to PSU channel %d\n', ch);
            input('Press Enter');
            if isDMM7510
                fprintf(myDmm, 'SENS:FUNC "VOLT:DC"\n');
                fprintf(myDmm, 'SENS:VOLT:RANG 32\n' );
                fprintf(myDmm, 'VOLT:NPLC %d\n' , myNplc)
                if myAverage > 1
                    fprintf(myDmm, 'VOLT:AVER:COUN %d\n' , myAverage);
                    fprintf(myDmm, 'VOLT:AVER ON\n' );
                else
                    fprintf(myDmm, 'VOLT:AVER OFF\n' );
                end                    
                fprintf(myDmm, 'VOLT:AZER OFF\n');
                query( myDmm , '*OPC?');
                fprintf(myDmm, 'AZER:ONCE\n');
                query( myDmm , '*OPC?');
            else
                fprintf(myDmm, 'CONF:VOLT:DC AUTO\n');
                fprintf(myDmm, 'VOLT:DC:NPLC %d\n', myNplc);
            end
            CheckVisaError(myDmm);

            fprintf(myPsu, 'OUTPUT CH%d,ON\n', ch);
            CheckVisaError(myPsu);

            fprintf('Running DAC-V for channel %d\n', ch);
            for i = 1:size(DACV, 2)
                myStep = i-1;
                str = sprintf('CALibration:Set CH%d,V,%d,%fV,1\n', ch, myStep, DACV(i));
                fprintf(myPsu, str );
                CheckVisaError(myPsu);
                pause(myPause);
                DmmV = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                str = sprintf('CALibration:MEAS CH%d,V,%d,%fV,1\n', ch, myStep, DmmV);
                fprintf(myPsu, str );
                CheckVisaError(myPsu);
                fprintf('DACV Ch:%d - Step:%d - Nom:%fV - DMM:%fV\n', ch, myStep, DACV(i), DmmV);
            end

            fprintf('Running ADC-V for channel %d\n', ch);
            for i = 1:size(ADCV, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,V,%d,%fV,0\n', ch, myStep, ADCV(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmV = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,V,%d,%fV,0\n', ch, myStep, DmmV));
                CheckVisaError(myPsu);
                fprintf('ADCV Ch:%d - Step:%d - Nom:%fV - DMM:%fV\n', ch, myStep, ADCV(i), DmmV);
            end

            fprintf(myPsu, 'OUTPUT CH%d,OFF\n', ch);
            CheckVisaError(myPsu);

            fprintf('Starting Current: connect the DMM 10A CURRENT inputs to PSU channel %d\n', ch);
            input('Press Enter');
            if isDMM7510
                fprintf(myDmm, 'SENS:FUNC "CURR:DC"\n');
                fprintf(myDmm, 'SENS:CURR:RANG 10\n' );
                fprintf(myDmm, 'CURR:NPLC %d\n' , myNplc)
                if myAverage > 1
                    fprintf(myDmm, 'CURR:AVER:COUN %d\n' , myAverage);
                    fprintf(myDmm, 'CURR:AVER ON\n' );
                else
                    fprintf(myDmm, 'CURR:AVER OFF\n' );
                end
                %Need to substantially increase Timeout for Auto Zero
                %commands to finish.
                myDmm.Timeout = 30;
                fprintf(myDmm, 'CURR:AZER OFF\n');
                query( myDmm , '*OPC?');
                fprintf(myDmm, 'AZER:ONCE\n');
                query( myDmm , '*OPC?');
                myDmm.Timeout = 10;
            else
                fprintf(myDmm, 'CONF:CURR:DC 10A\n'); %Sets input to 10A port
                fprintf(myDmm, 'CURR:DC:NPLC %d\n', myNplc);
            end
            
            CheckVisaError(myDmm);

            fprintf(myPsu, 'OUTPUT CH%d,ON\n', ch);
            CheckVisaError(myPsu);

            fprintf('Running DAC-I for channel %d\n', ch);
            for i = 1:size(DACI, 2)
                myStep = i-1;
                str =  sprintf('CALibration:Set CH%d,C,%d,%fA,1\n', ch, myStep, DACI(i));
                fprintf(myPsu,str);
                CheckVisaError(myPsu);
                pause(myPause);
                DmmI = str2double(query(myDmm,'READ?'));
                query( myDmm , '*OPC?' );
                CheckVisaError(myDmm);
                str = sprintf('CALibration:MEAS CH%d,C,%d,%fA,1\n', ch, myStep, DmmI);
                fprintf(myPsu, str );
                CheckVisaError(myPsu);
                fprintf('DACI Ch:%d - Step:%d - Nom:%fA - DMM:%fA\n', ch, myStep, DACI(i), DmmI);
            end

            fprintf('Running ADC-I for channel %d\n', ch);
            for i = 1:size(ADCI, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,C,%d,%fA,0\n', ch, myStep, ADCI(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmI = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,C,%d,%fA,0\n', ch, myStep, DmmI));
                CheckVisaError(myPsu);
                fprintf('ADCI Ch:%d - Step:%d - Nom:%fA - DMM:%fA\n', ch, myStep, ADCI(i), DmmI);
            end

            fprintf(myPsu, 'OUTPUT CH%d,OFF', ch);
            CheckVisaError(myPsu);
            if simOnly
                fprintf('SKIPPING STORING CALIBRATION\n');
                fprintf('Change the "simOnly" variable to false when you want to proceed with actual calibration\n');
            else
                fprintf('Writing calibration channel %d\n', ch);
                fprintf(myPsu, sprintf('CALibration:End %s,CH%d\n', datestr(now,'mm/dd/yyyy'), ch));
                CheckVisaError(myPsu);
            end

            fprintf('Calibration DONE channel %d\n', ch);
        end

        fprintf('\nCALIBRATION DONE\n');
        fclose(myDmm);
        fclose(myPsu);

    catch MExc
        MExc.message
        delete_all_objs()

    end
end

function delete_all_objs
%DELETE ALL OBJS
%Usage
%  Clean up all instrument objects not properly closed.
%  Run once at beginning of script or at the end to clean up objects
%
    newobjs = instrfind;
    if isempty(newobjs) == 0 
      % if using device objects may need to disconnect instead of close
      % disconnect(newobjs)
      fclose(newobjs);
      delete(newobjs);
      clear newobjs
    end
end

function VisaErr = CheckVisaError(myVisa)
%CHECK VISA ERROR
%Usage
%  Checks SCPI SYST:ERR? to make sure it says some version of 'No Error'.
%  If it doesn't, repeat until it does, and throw an esxception with all
%  the logged errors.
%  Works for both 34461A and DP832
%
    fprintf(myVisa, 'SYST:ERR?\n');
    errStr = fscanf(myVisa);

    if (strfind(errStr, 'No error')) %If no error, then return
        return;
        %If there is an error, read out all of the errors and return them in an exception
    else
        errStr2 = errStr;
        NoErr = strfind(errStr2, 'No error');
        while (isempty(NoErr));
            fprintf(myVisa, 'SYST:ERR?\n');
            errStr2 = fscanf(myVisa);
            NoErr = strfind(errStr2, 'No error');
            if (isempty(NoErr))
                errStr = strcat(errStr, '\n', errStr2);
            end
        end
        errStr = strcat('Exception: Encountered system error(s)\n', errStr)
        err = MException('ResultChk:BadInput', errStr);
        throw(err)
    end
end    


% NOTE: This script will NOT store the calibration into the PSU at the end.
% If this is not your intent, please abort and modify the value of the  "simOnly" variable.
% 
% PSU: RIGOL TECHNOLOGIES,DP832,DP8C163952907,00.01.14
% 
% DMM: KEITHLEY INSTRUMENTS,MODEL DMM7510,04079393,1.3.0s
% 
% Remove all leads from DMM and PSU to run self-tests
% Press Enter
% 
% Running PSU board self-test...
% PSU self-test result: PASS,PASS
% PSU internal temp: 29.8C
% 
% Running DMM self-test...
% DMM self-test result: 0 (0 = PASS, 1=FAIL)
% DMM internal temp: 32.5C
% 
% Starting channel 3
% 
% Starting Voltage: connect the DMM VOLTAGE inputs to PSU channel 3
% Press Enter
% Running DAC-V for channel 3
% DACV Ch:3 - Step:0 - Nom:0.100000V - DMM:0.050176V
% DACV Ch:3 - Step:1 - Nom:0.200000V - DMM:0.149518V
% DACV Ch:3 - Step:2 - Nom:0.400000V - DMM:0.348992V
% DACV Ch:3 - Step:3 - Nom:0.850000V - DMM:0.794342V
% DACV Ch:3 - Step:4 - Nom:1.200000V - DMM:1.140009V
% DACV Ch:3 - Step:5 - Nom:1.800000V - DMM:1.739363V
% DACV Ch:3 - Step:6 - Nom:2.550000V - DMM:2.483572V
% DACV Ch:3 - Step:7 - Nom:3.100000V - DMM:3.031143V
% DACV Ch:3 - Step:8 - Nom:3.400000V - DMM:3.327541V
% DACV Ch:3 - Step:9 - Nom:4.100000V - DMM:4.020601V
% DACV Ch:3 - Step:10 - Nom:4.500000V - DMM:4.418170V
% DACV Ch:3 - Step:11 - Nom:5.000000V - DMM:4.914656V
% DACV Ch:3 - Step:12 - Nom:5.300000V - DMM:5.210928V
% Running ADC-V for channel 3
% ADCV Ch:3 - Step:0 - Nom:0.000000V - DMM:0.000114V
% ADCV Ch:3 - Step:1 - Nom:0.005000V - DMM:0.005088V
% ADCV Ch:3 - Step:2 - Nom:0.010000V - DMM:0.010066V
% ADCV Ch:3 - Step:3 - Nom:0.020000V - DMM:0.020067V
% ADCV Ch:3 - Step:4 - Nom:0.050000V - DMM:0.049937V
% ADCV Ch:3 - Step:5 - Nom:0.100000V - DMM:0.099788V
% ADCV Ch:3 - Step:6 - Nom:0.500000V - DMM:0.499951V
% ADCV Ch:3 - Step:7 - Nom:1.000000V - DMM:0.999459V
% ADCV Ch:3 - Step:8 - Nom:3.000000V - DMM:3.000270V
% ADCV Ch:3 - Step:9 - Nom:5.000000V - DMM:4.999937V
% ADCV Ch:3 - Step:10 - Nom:5.300000V - DMM:5.300381V
% Starting Current: connect the DMM 10A CURRENT inputs to PSU channel 3
% Press Enter
% Running DAC-I for channel 3
% DACI Ch:3 - Step:0 - Nom:0.100000A - DMM:0.038728A
% DACI Ch:3 - Step:1 - Nom:0.500000A - DMM:0.435923A
% DACI Ch:3 - Step:2 - Nom:1.000000A - DMM:0.932698A
% DACI Ch:3 - Step:3 - Nom:1.250000A - DMM:1.181775A
% DACI Ch:3 - Step:4 - Nom:1.500000A - DMM:1.430789A
% DACI Ch:3 - Step:5 - Nom:1.750000A - DMM:1.680669A
% DACI Ch:3 - Step:6 - Nom:2.000000A - DMM:1.930447A
% DACI Ch:3 - Step:7 - Nom:2.250000A - DMM:2.179178A
% DACI Ch:3 - Step:8 - Nom:2.500000A - DMM:2.427539A
% DACI Ch:3 - Step:9 - Nom:2.750000A - DMM:2.676377A
% DACI Ch:3 - Step:10 - Nom:3.000000A - DMM:2.924734A
% DACI Ch:3 - Step:11 - Nom:3.200000A - DMM:3.122717A
% Running ADC-I for channel 3
% ADCI Ch:3 - Step:0 - Nom:0.000000A - DMM:-0.000132A
% ADCI Ch:3 - Step:1 - Nom:0.100000A - DMM:0.100114A
% ADCI Ch:3 - Step:2 - Nom:1.000000A - DMM:1.000322A
% ADCI Ch:3 - Step:3 - Nom:2.000000A - DMM:2.000304A
% ADCI Ch:3 - Step:4 - Nom:3.000000A - DMM:3.000067A
% ADCI Ch:3 - Step:5 - Nom:3.200000A - DMM:3.199656A
% SKIPPING STORING CALIBRATION
% Change the "simOnly" variable to false when you want to proceed with actual calibration
% Calibration DONE channel 3

